@@ -18,7 +18,7 @@ NULL
 #' @family objectives
 #'
 #' @examples
-#' \donttest{
+#' \dontrun{
 #' # load data
 #' habitat_data <- rast(
 #'   system.file("extdata", "habitat_hi_res.tif", package = "restoptr")
@@ -67,11 +67,11 @@ set_min_restore_objective <- function(problem) {
       name = "Minimize restoration area",
       class = c("MinRestoreObjective", "RestoptObjectve"),
       post = function(jproblem, nb_solutions, precision, time_limit, optimality_gap,
-                      verbose=FALSE, search_strategy="") {
+                      verbose=FALSE, search_strategy="", lns=FALSE) {
         rJava::.jcall(
           jproblem, "Ljava/util/List;", "minimizeMinRestore",
           get_habitat_threshold(problem), nb_solutions, time_limit, optimality_gap,
-          verbose, search_strategy
+          verbose, search_strategy, lns
         )
       }
     )
